//pins.cpp
#include "t3216_puzzle_code.h"
#include "pins.h"

// //The 6 pins attached to puzzle pieces on the ATtiny3216
// #define PIN_PUZZLE_1 1
// #define PIN_PUZZLE_2 2
// #define PIN_PUZZLE_3 3
// #define PIN_PUZZLE_4 4
// #define PIN_PUZZLE_5 5
// #define PIN_PUZZLE_6 6

// //The interrupt pins of the PCA9555Ds
// #define PIN_INT_0x20 7
// #define PIN_INT_0x21 10

//Setup the two PCA9555D  boards
PCA9555 board_0x20(0x20); //16 GPIO pins
PCA9555 board_0x21(0x21); //16 GPIO pins
// PCA9555 board_0x20(0x20, PIN_INT_0x20); //16 GPIO pins
// PCA9555 board_0x21(0x21, PIN_INT_0x21); //16 GPIO pins

//Saves the states of each pin
byte pin_state[num_pieces];
byte pin_state_old[num_pieces];
byte pieces_placed = 0;

////////////////////////////////////////////////////////
///////// Check if any pins have changed state /////////
////////////////////////////////////////////////////////
void checkPiecePlaced(bool keep_once_placed) {
  const byte debounce_delay = 150;

  for(byte p = 0; p < num_pieces; p++) {

    //If keep_once_placed == true, only check for those that have never before been changed
    if(keep_once_placed == false | piece_matrix[p].piece_state == 1) {
      byte index = piece_matrix[p].pin_index;

      //Debounce checks
      if(pin_state[index] != pin_state_old[index]) {
        //Reset the debouncing timer
        piece_matrix[p].last_debounce_time = millis();
      }//if

      //Only step inside if enough time has passed with the same state
      if ((millis() - piece_matrix[p].last_debounce_time) > debounce_delay) {
        //If the state has changed
        if (pin_state[index] != piece_matrix[p].piece_state) {
          piece_matrix[p].piece_state = pin_state[index];
          piece_matrix[p].piece_placed = !piece_matrix[p].piece_placed;
        }//if
      }//if
    }//if

  }//for p

  for(byte p = 0; p < num_pieces; p++) {
    pin_state_old[p] = pin_state[p];
  }//for p
}//void checkPiecePlaced

////////////////////////////////////////////////////////
////////// Check the current state of each pin /////////
////////////////////////////////////////////////////////
void checkPins() {
  byte n = 0;
  byte i;
  byte state_current;

  //Check pin states of ATtiny
  for (i = 1; i <= 6; i++) {
    state_current = digitalRead(i);
    checkPinState(state_current, n, i, "ATtiny");
    n++;
  }//for i

  //Check pin states of 0x20 board
  for (i = 0; i < 16; i++) {
    state_current = board_0x20.digitalRead(i);
    // Serial.print(state_current);
    checkPinState(state_current, n, i, "0x20");
    n++;
  }//for i

  //Check pin states of 0x21 board
  for (i = 0; i < 16; i++) {
    state_current = board_0x21.digitalRead(i);
    // Serial.print(state_current);
    checkPinState(state_current, n, i, "0x21");
    n++;
  }//for i

  // Serial.println("");
}//void checkPins

////////////////////////////////////////////////////////
///////////////// Save if state changed ////////////////
////////////////////////////////////////////////////////
void checkPinState(byte state, byte n, byte i, char *board) {
  if(state != pin_state[n]) {
    //Print change to Serial Monitor
    if(Serial) {
      Serial.print(board);
      Serial.print(F(" board: "));
      Serial.print(i);
      Serial.print(F(" Piece ID: "));
      Serial.println(puzzle_id_map[n]);
    }//if

    //Save in array
    pin_state[n] = state;
  }//if
}//void checkPinState

